<?php
class WeatherIconConfig
{
	function init()
	{
		global $wp_version;

		// Add Menu under options
		$page = add_options_page( 
			__('WeatherIcon Options', 'WI'),
			__('WeatherIcon', 'WI'),
			8,
			basename(__FILE__),
			array('WeatherIconConfig', 'WI_options_page'));

        if (version_compare($wp_version, '2.1', '>='))
        {
            add_action ("admin_print_scripts-$page", array ('WeatherIconConfig', 'js_includes'));
            add_action ("admin_head-$page", array ('WeatherIconConfig', 'WI_admin_head'), 50);
        }
        else
        {   
            add_action ("admin_head", array ('WeatherIconConfig', 'WI_admin_head'), 50);
        }
	}

	function js_includes()
	{
		wp_enqueue_script('prototype');
		wp_enqueue_script('scriptaculous');
		wp_enqueue_script('WIconfigscreen', WIjsdirWeb . 'ConfigurationScreen.js.php', array('prototype','scriptaculous'), WIversion);
		wp_enqueue_script('mfLightbox', WIjsdirWeb . 'mf_lightbox.js', array('prototype'), '');
	}

	function WI_admin_head()
	{
		global $wp_version;

		echo "\n<!-- WeatherIcon Options CSS and Javascript -->\n";
		echo '<link rel="stylesheet" href="' . WIdirWeb . '/css/ConfigurationScreen.css" type="text/css" screen="media" >'."\n";
		echo '<link rel="stylesheet" href="' . WIdirWeb . '/css/mf_lightbox/lightbox.css" type="text/css" screen="media" >'."\n";

		// If this isn't wordpress 2.1+, we need to output our JS scripts
		if (version_compare($wp_version, '2.1', '<'))
		{
			echo '<script src="' . WIjsdirWeb . 'scriptaculous/prototype.js" type="text/javascript"></script>'."\n";
			echo '<script src="' . WIjsdirWeb . 'scriptaculous/scriptaculous.js" type="text/javascript"></script>'."\n";
			echo '<script src="' . WIjsdirWeb . 'ConfigurationScreen.js.php" type="text/javascript"></script>'."\n";
			echo '<script src="' . WIjsdirWeb . 'mf_lightbox.js" type="text/javascript"></script>'."\n";
		}

		echo '<!-- End WeatherIcon Options CSS and Javascript -->' . "\n\n";
	}
	
	function WI_options_page()
	{
		global $WI, $wp_version, $l10n;

		if (isset($_POST['WIreset']))
		{
			update_option('WIversion', '0.0.0');
			delete_option('WIsettings');
			$WI->checkForUpgrade();
			$WI->updateSettings();

			// Unset the current Text Domain
			unset($l10n['WI']);

			// Load the default TextDomain
			load_textdomain('WI', WIi18ndir . $WI->settings['language'] . ".mo");
		}

		if (isset($_POST['WIclear_cache']))
		{
			$WI->clearMetarCacheTable();
		}
		
		if (isset($_POST['WIsubmit']))
		{
			parse_str($_POST['display_options']);

			foreach ($active as $value)
			{
				$_POST['WIsettings'][display_options][] = "display_".$value;
			}

			/*
			** This line is a work-around for a bug in WordPress function update_option().
			** As of version 2.0.5, this function incorrectly identifies whether the array has changed.
			** See bug http://trac.wordpress.org/ticket/3394
			** Note that this will force a database write on form submit, even if no data has changed.
			** Once WP is upgraded, we can put a version check around this line (or remove it completely
			** if WP is upgraded before this version of WeatherIcon is released).
			*/
			$_POST['WIsettings'][timestamp] = time();

			update_option('WIsettings', $_POST['WIsettings']);
			$WI->updateSettings();

			// Unset the current Text Domain
			unset ($l10n['WI']);

			// Load the TextDomain new? language?
			load_textdomain('WI', WIi18ndir . $WI->settings['language'] . ".mo");

			 echo '<div class="updated"><p><strong>' . __('Settings Updated', 'WI') . '</strong></p></div>';
		}

		if ($WI->version_num(WIversion) < $WI->CheckForNewVersion())
		{
			echo '<div class="updated fade" id="message">';
			echo '<p><a href="' . $WI->getUpdateURL() .'">' . __('A new version of WeatherIcon available', 'WI') . '</a></p>';
			echo '</div>';
		}

		// Configuration Page
		echo '<div class="wrap">'."\n";
		echo '<h2>' . __('WeatherIcon Options', 'WI') . '</h2>'."\n";
		echo '<form name="weathericon_options" method="post" action="" onSubmit="SerializeDisplayOptions();">'."\n";
		echo '<div id="WI_Options_Left">'."\n";
		echo '<fieldset class="options">'."\n";
		echo '<legend>' . __('WeatherIcon Info', 'WI') . '</legend>'."\n";
		echo '<table class="optiontable">'."\n";
		echo '<tbody><tr>'."\n";
		echo '<th scope="row">' . __('Version', 'WI') . ':</th>'."\n";
		echo '<td>' . WIversion . '</td>'."\n";
		echo '</tr><tr>'."\n";
		echo '<th scope="row">' . __('Wordpress Version', 'WI') . ':</th>'."\n";
		echo '<td>' . $wp_version . '</td>'."\n";
		echo '</tr><tr>'."\n";
		echo '<th scope="row">' . __('PHP version', 'WI') . ':</th>'."\n";
		echo '<td>' . phpversion() . '</td>'."\n";
		echo '</tr></tbody>'."\n";
		echo '</table>'."\n";
		echo '</fieldset>'."\n";

		$display_list = array (
			'display_locname'			=> __('Location Name', 'WI'),
			'display_icon'				=> __('Icon', 'WI'),
			'display_temp'				=> __('Temperature', 'WI'),
			'display_heatindex'		 	=> __('Heat Index', 'WI'),
			'display_windchill'			=> __('Wind Chill', 'WI'),
			'display_humidex'			=> __('Humidex', 'WI'),
			'display_dewpt'				=> __('Dew Point', 'WI'),
			'display_humidity'			=> __('Humidity', 'WI'),
			'display_barometer'			=> __('Barometer', 'WI'),
			'display_wind'				=> __('Wind', 'WI'),
			'display_clouds'			=> __('Clouds (Simple)', 'WI'),
			'display_cloudsdetailed'	=> __('Clouds (Complex)', 'WI'),
			'display_conditions'		=> __('Conditions', 'WI'),
			'display_visibility'		=> __('Visibility', 'WI'),
			'display_sunrise'			=> __('Sunrise', 'WI'),
			'display_sunset'			=> __('Sunset', 'WI'),
			'display_updatetime'		=> __('Update Time', 'WI')
		);

		$active = '';
		foreach ($WI->settings['display_options'] as $value)
		{
			$display_list_keys = array_keys($display_list);
			if (in_array($value, $display_list_keys))
			{
				$active .= '<li name="' . $value . '" id="' . $value . '">' . $display_list[$value] . '</li>'."\n";

				// Remove displayed element from list
				$loc = array_search($value, $display_list_keys);
				$display_list = array_merge(
					array_slice($display_list, 0, $loc),
					array_slice($display_list, $loc + 1)
				);
			}
		}

		$disabled = '';
		foreach ($display_list as $idx => $value)
		{
			$disabled .= '<li name="' . $value . '" id="' . $idx . '">' . $value . '</li>'."\n";
		}

		echo '<fieldset id="weather-options"><legend>' . __('Display Options', 'WI') . '</legend>'."\n";
		echo '<table id="WI_DisplayOptions">'."\n";
		echo '<thead>'."\n";
		echo '<tr valign="bottom">'."\n";
		echo '<th>' . __('Display Order', 'WI') . '</th>'."\n";
		echo '<th>' . __('Disabled<br />Display Options', 'WI') . '</th>'."\n";
		echo '</tr>'."\n";
		echo '</thead>'."\n";
		echo '<tbody>'."\n";
		echo '<tr valign="top">'."\n";
		echo '<td id="active-list"><ul id="active" class="boxy">' . $active . '</ul></td>'."\n";
		echo '<td id="disabled-list"><ul id="disabled" class="boxy">' . $disabled . '</ul></td>'."\n";
		echo '</tr></tbody></table>'."\n";
		echo '<input type="hidden" id="display_options" name="display_options" value="" />';
		echo '</fieldset></div>'."\n";

		//echo '<div id="display_options"></div>'."\n";

		echo '<script type="text/javascript">'."\n";
		echo '// <![CDATA['."\n";
		echo 'Sortable.create("active", {dropOnEmpty:true,containment:["active","disabled"],constraint:false});'."\n";
		echo 'Sortable.create("disabled", {dropOnEmpty:true,containment:["active","disabled"],constraint:false});'."\n";
		echo '// ]]>'."\n";
		echo '</script>'."\n";

		echo '<div id="WI_Options_Right">'."\n";
		echo '<fieldset class="options">'."\n";
		echo '<legend>' . __('General Options', 'WI') . '</legend>'."\n";
		echo '<table class="optiontable3">'."\n";
		echo '<tbody><tr >'."\n";
		echo '<th scope="row">' . __('Language', 'WI') . ':</th>'."\n";
		echo '<td colspan="3"><select name="WIsettings[language]">' . WeatherIconConfig::get_i18n_options('language') . '</select></td>'."\n";
// Very Very broken.   Don't let people use until fixed!! -GH
//		echo '<th scope="row" width="250">' . __('Add Weather Data to Posts?', 'WI') . '</th>'."\n";
//		echo '<td><input name="WIsettings[display_options][]" value="show_current_weather_at_post" ' . $WI->option_checked('show_current_weather_at_post') . ' type="checkbox" /></td>'."\n";
		echo '</tr><tr >'."\n";
		echo '<th scope="row">' . __('Cache Timeout', 'WI') . ':</th>'."\n";
		echo '<td><input name="WIsettings[cache_timeout]" size="6" class="code" type="text" value="' . $WI->settings['cache_timeout'] . '" /></td>'."\n";

		echo '</tr></tbody>'."\n";
		echo '</table>'."\n";
		echo '</fieldset>'."\n";

		echo '<fieldset class="options">'."\n";
		echo '<legend>' . __('Icon Pack', 'WI') . '</legend>'."\n";
		echo '<table class="optiontable2">'."\n";
		echo '<thead>'."\n";
		echo '<tr>'."\n";
		echo '<td colspan="4">'. __('To acquire more icon packs, <a href="http://www.viper007bond.com/wordpress-plugins/forums/viewtopic.php?id=302">go here</a>', 'WI') . '</td>'."\n";
		echo '</tr>'."\n";
		echo '</thead>'."\n";
		echo '<tbody>'."\n";
		echo '<tr>'."\n";
		echo '<th scope="row">' . __('Example', 'WI') . ':</th>'."\n";
		echo '<td class="icon-sample"><img src="' . WIdirWeb . "/ipkfile.php?pack=" . $WI->settings['icon_pack'] . '&amp;icon=' . current(array_keys($WI->icon_packs[$WI->settings['icon_pack']]['icons'])) . '&amp;type=' . $WI->icon_packs[$WI->settings['icon_pack']]['formats']['format'][$WI->settings['icon_ext']] . '" id="previewimg" alt="Sample icon" /></td>'."\n";
		echo '<th scope="row">' . __('Icon Format', 'WI') . ':</th><td><select name="WIsettings[icon_ext]" id="WIsettings[icon_ext]" style="width: 5em;">'."\n";
		foreach ($WI->icon_packs[$WI->settings['icon_pack']]['formats']['format'] as $idx => $format)
		{
			echo '<option value="' . $idx . '" ' . $WI->setting_selected('icon_ext', $idx) . '>' . $format . '</option>'."\n";
		}
		echo '</select></td>'."\n";
		echo '</tr></tbody>'."\n";
		echo '</table>'."\n";
		echo '<table class="listtable" id="iconpacks">'."\n";
		echo '<thead>'."\n";
		echo '<tr>'."\n";
		echo '<th scope="col">&nbsp;</th>'."\n";
		echo '<th scope="col">'. __('Icon Pack', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Designer', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Description', 'WI') . '</th>'."\n";
		echo '</tr>'."\n";
		echo '</thead>'."\n";

		echo '<tbody class="icons">'."\n";
		foreach ($WI->icon_packs as $name=>$icon_pack)
		{
			$title = $icon_pack['title'];

			if (!empty($icon_pack['designer']['uri']))
			{
				$designer = '<a href="' . $icon_pack['designer']['uri'] . '" title="' . __('Visit designer homepage', 'WI') . '">' . $icon_pack['designer']['name'] . '</a>';
			}
			else
			{
				$designer = $icon_pack['designer']['name'];
			}

			$description = $icon_pack['description'];
			$i = $i + 1;
			$class = ($i % 2 == 0) ? '':' class="alternate"';
			
			echo '<tr id="' . $name . '"' . $class .'>'."\n";
			echo '<th class="togl"><input type="radio" name="WIsettings[icon_pack]" value="' . $name . '" ' . $WI->setting_checked('icon_pack', $name) . ' onClick="updateForm(\''. $name .'\');" /></th>'."\n";
			if (!empty($icon_pack['license']))
			{
				echo '<div style="display:none;" id="' . $name . '_license">' . $icon_pack['license'] . '</div>';
				echo '<td class="name"><a href="#" class="license" onClick="Lightbox.showBoxByID(\''. $name . '_license\', \'600\', \'400\');" title="Usage License Ageement">' . $title . '</a></td>'."\n";
			}
			else
			{
				echo '<td class="name">' .  $title . '</td>'."\n";
			}
			echo '<td class="auth">' . $designer . '</td>'."\n";
			echo '<td class="desc">' . $description . '</td>'."\n";
			echo '</tr>'."\n";
		}
		echo '</tbody>'."\n";
		echo '</table>'."\n";
		echo '<table class="optiontable">'."\n";
        echo '<tbody>'."\n";
        echo '<tr>'."\n";
        echo '<td colspan="4">' . __('Report Icon Errors to developers?', 'WI');
        echo ' <input name="WIsettings[display_options][]" value="allow_report" ' . $WI->option_checked('allow_report') . ' type="checkbox" /></td>'."\n";
        echo '</tr>'."\n";
		echo '</tbody>'."\n";
		echo '</table>'."\n";
		echo '</fieldset>'."\n";
		echo '</div>'."\n";

		echo '<div id="WI_Options_Bottom">'."\n";
		echo '<fieldset class="options"><legend>' . __('Date/Time Settings', 'WI') . '</legend>'."\n";
		echo '<table class="optiontable3">'."\n";
		echo '<tr >'."\n";
		echo '<th scope="row">' . __('Use', 'WI') . ':</th>'."\n";
		echo '<td colspan="3"><select name="WIsettings[usetime]" title="' . __('The selection is only used for determining whether it is night or not.', 'WI') . '">'."\n";
		echo '<option value="currenttime" ' . $WI->setting_selected ('usetime', 'currenttime') . '>' . __('Current Time', 'WI') .'</option>'."\n";
		echo '<option value="metartime" '. $WI->setting_selected ('usetime', 'metartime') . '>' . __('Metar Time', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
// Obsolete now
// -- Garett
//		echo '<th scope="row">' . __('GMT Offset', 'WI') . ':</th>'."\n";
//	  echo '<td><input name="WIsettings[gmt_offset]" value="' . $WI->settings['gmt_offset'] . '" size="10" type="text" /></td>'."\n";
		echo '</tr><tr >'."\n";
// No longer used
// -- Garett
//		echo '<th scope="row">' . __('Sunrise Hour', 'WI') . ':</th>'."\n";
//		echo '<td><input name="WIsettings[sunrise_hour]" size="6" class="code" type="text" value="' . $WI->settings['sunrise_hour'] . '" /></td>'."\n";
		echo '<th scope="row">' . __('Date Format', 'WI') . ':</th>'."\n";
		echo '<td><input name="WIsettings[default_date]" size="10" class="code" type="text" value="' . $WI->settings['default_date'] . '" /></td>';
		echo '<th scope="row">' . __('Time Format', 'WI') . ':</th>'."\n";
		echo '<td class="wide"><input name="WIsettings[default_time]" size="10" class="code" type="text" value="' . $WI->settings['default_time'] . '" /></td>'."\n";
		echo '</tr><tr >'."\n";
// No longer used
// -- Garett
//		echo '<th scope="row">' . __('Sunset Hour', 'WI') . ':</th>'."\n";
//		echo '<td><input name="WIsettings[sunset_hour]" size="6" class="code" type="text" value="' . $WI->settings['sunset_hour'] . '" /></td>'."\n";
		echo '<td colspan="4" align="center"><a href="http://codex.wordpress.org/Formatting_Date_and_Time">' . __('Documentation on date/time formatting.', 'WI') . '</a></td>'."\n";
		echo '</tr></table></fieldset>'."\n";


		echo '<fieldset class="options"><legend>' . __('Units of Measure', 'WI') . '</legend>'."\n";
		echo '<table class="optiontable3">'."\n";
		echo '<tr >'."\n";
		echo '<th scope="row">' . __('Temperature Units', 'WI') . ':</th>'."\n";
		echo '<td><select name="WIsettings[units][temp]">'."\n";
		echo '<option value="C" ' . $WI->unit_selected('temp', 'C') . '>' . __('Celsius', 'WI') . '</option>'."\n";
		echo '<option value="F" ' . $WI->unit_selected('temp', 'F') . '>' . __('Fahrenheit', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
		echo '<th scope="row">' . __('Visibility Units:', 'WI') . '</th>'."\n";
		echo '<td class="wide"><select name="WIsettings[units][visibility]">'."\n";
		echo '<option value="m" ' . $WI->unit_selected('visibility', 'm') . '>' . __('Meters', 'WI') . '</option>'."\n";
		echo '<option value="km" ' . $WI->unit_selected('visibility', 'km') . '>' . __('Kilometers', 'WI') . '</option>'."\n";
		echo '<option value="ft" ' . $WI->unit_selected('visibility', 'ft') . '>' . __('Feet', 'WI') . '</option>'."\n";
		echo '<option value="miles" ' . $WI->unit_selected('visibility', 'miles') . '>' . __('Miles', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
		echo '</tr>'."\n";
		echo '<tr >'."\n";
		echo '<th scope="row">' . __('Wind Units', 'WI') . ':</th>'."\n";
		echo '<td><select name="WIsettings[units][wind]">'."\n";
		echo '<option value="km/h" ' . $WI->unit_selected('wind', 'km/h') . '>' . __('Kilometers per Hour', 'WI') . '</option>'."\n";
		echo '<option value="m/s" ' . $WI->unit_selected('wind', 'm/s') . '>' . __('Meters per Second', 'WI') . '</option>'."\n";
		echo '<option value="mph" ' . $WI->unit_selected('wind', 'mph') . '>' . __('Miles per Hour', 'WI') . '</option>'."\n";
		echo '<option value="kt" ' . $WI->unit_selected('wind', 'kt') . '>' . __('Knots', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
		echo '<th scope="row">' . __('Cloud Height Units:', 'WI') . '</th>'."\n";
		echo '<td><select name="WIsettings[units][clouds]">'."\n";
		echo '<option value="m" ' . $WI->unit_selected('clouds', 'm') . '>' . __('Meters', 'WI') . '</option>'."\n";
		echo '<option value="km" ' . $WI->unit_selected('clouds', 'km') . '>' . __('Kilometers', 'WI') . '</option>'."\n";
		echo '<option value="ft" ' . $WI->unit_selected('clouds', 'ft') . '>' . __('Feet', 'WI') . '</option>'."\n";
		echo '<option value="miles" ' . $WI->unit_selected('clouds', 'miles') . '>' . __('Miles', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
		echo '</tr>'."\n";
		echo '<tr >'."\n";
		echo '<th scope="row">' . __('Pressure Units:', 'WI') . '</th>'."\n";
		echo '<td><select name="WIsettings[units][pressure]">'."\n";
		echo '<option value="hPa" ' . $WI->unit_selected('pressure', 'hPa') . '>' . __('Hectopascals (hPa)', 'WI') . '</option>'."\n";
		echo '<option value="atm" ' . $WI->unit_selected('pressure', 'atm') . '>' . __('Atmospheres (atm)', 'WI') . '</option>'."\n";
		echo '<option value="inHg" ' . $WI->unit_selected('pressure', 'inHg') . '>' . __('Inches of Mercury (inHg)', 'WI') . '</option>'."\n";
		echo '<option value="mmHg" ' . $WI->unit_selected('pressure', 'mmHg') . '>' . __('Torrs (mmHg)', 'WI') . '</option>'."\n";
		echo '</select></td>'."\n";
		echo '</tr>'."\n";
		echo '</table>'."\n";
		echo '</fieldset>'."\n";

		echo '<fieldset class="options"><legend>' . __('Preset Locations', 'WI') . '</legend>'."\n";
		echo '<table class="listtable">'."\n";
		echo '<thead><tr>'."\n";
		echo '<th scope="col">' . __('Location Name', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Station', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Latitude', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Longitude', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('GMT Offset', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('DST?', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Language', 'WI') . '</th>'."\n";
		echo '<th scope="col">' . __('Action', 'WI') . '</th>'."\n";
		echo '</tr>'."\n";
		echo '</thead>'."\n";
		echo '<tbody id="locations">'."\n";

		$locations = '';
		foreach ($WI->settings['locations'] as $loc_name => $loc_info)
		{
				$count++;
				if ($WI->settings['default_location'] == $loc_name)
				{
					$alternate = ($count %  2) == 1 ? ' class="alternate active"' : ' class="active"';
				}
				else
				{
					$alternate = ($count %  2) == 1 ? ' class="alternate"' : '';
				}
				$locations .= '<tr' . $alternate . ' id="entry-' . $loc_name . '">'."\n";
				$locations .= '<th class="left">' . $loc_name . '</th>'."\n";
				$locations .= '<td><input type="text" name="WIsettings[locations][' . $loc_name . '][station]" value="' . $loc_info['station'] . '" size="5" maxlength="4" /></td>'."\n";
				$locations .= '<td><input type="text" name="WIsettings[locations][' . $loc_name . '][latitude]" value="' . $loc_info['latitude'] . '" size="10" maxlength="9" /></td>'."\n";
				$locations .= '<td><input type="text" name="WIsettings[locations][' . $loc_name . '][longitude]" value="' . $loc_info['longitude'] . '" size="10" maxlength="9" /></td>'."\n";
				$locations .= '<td><input type="text" name="WIsettings[locations][' . $loc_name . '][gmt_offset]" value="' . $loc_info['gmt_offset'] . '" size="4" maxlength="3" /></td>'."\n";
				$locations .= '<td><input  type="checkbox" name="WIsettings[locations][' . $loc_name . '][dst]" ';
				if ($loc_info['dst']) $locations .= 'checked="checked" ';
				$locations .= '/></td>'."\n";
				$locations .= '<td><select name="WIsettings[locations][' . $loc_name . '][language]">' . WeatherIconConfig::get_i18n_options('locations|' . $loc_name . '|language') . '</select></td>'."\n";
				if ($WI->settings['default_location'] == $loc_name)
				{
					$locations .= '<td id="actions-' . $loc_name . '" class="left">' ."\n". '<a href="javascript:show_debug(\'' . urlencode($loc_name) . '\');">Debug</a>' ."\n". '</td>'."\n";
				}
				else
				{
						$locations .= '<td id="actions-' . $loc_name . '" class="left">' ."\n". '<a href="javascript:show_debug(\'' . urlencode($loc_name) . '\');">Debug</a><br/>' ."\n". '<a href="javascript:make_default_loc(\'' . $loc_name . '\');">Make Default</a><br/>' ."\n". '<a href="javascript:delete_loc(\'' . $loc_name . '\');">Delete</a>' ."\n". '</td>'."\n";
				}
				$locations .= '</tr>'."\n";
		}
		echo "\n".$locations."\n";
		echo '</tbody>'."\n";

		echo '<script type="text/javascript">'."\n";
		echo '// <![CDATA['."\n";
		echo 'Sortable.create("locations", {tag:"tr",dropOnEmpty:true,constraint:false});'."\n";
		echo '// ]]>'."\n";
		echo '</script>'."\n";

		echo '<tfoot>'."\n";
		echo '<tr valign="bottom">'."\n";
		echo '<th>'.__('New Location', 'WI') . '</th>';
		echo '<td colspan="7">&nbsp;</td>'."\n";
		echo '</tr><tr>'."\n";
		echo '<th><input type="hidden" id="new_loc_alt" name="new_loc_alt" value="' . (($count+1)%2) . '"/><input type="text" id="new_loc_name" name="new_loc_name" value="" /></th>'."\n";
		echo '<td><input type="text" id="new_loc_station" name="new_loc_station" value="" size="5" maxlength="4" /></td>'."\n";
		echo '<td><input type="text" id="new_loc_latitude" name="new_loc_latitude" value="" size="10" maxlength="9" onkeypress="return onKeyPressBlockInvalidEntry_LatLong(event);" /></td>'."\n";
		echo '<td><input type="text" id="new_loc_longitude" name="new_loc_longitude" value="" size="10" maxlength="9" onkeypress="return onKeyPressBlockInvalidEntry_LatLong(event);" /></td>'."\n";
		echo '<td><input type="text" id="new_loc_gmt_offset" name="new_loc_gmt_offset" value="" size="4" maxlength="3" onkeypress="return onKeyPressBlockInvalidEntry_GMT(event);" /></td>'."\n";
		 echo '<td><input type="checkbox" id="new_loc_dst" name="new_loc_dst" checked="true" /></td>'."\n";
		echo '<td><select id="new_loc_language" name="new_loc_language">' . WeatherIconConfig::get_i18n_options('language') . '</select></td>'."\n";
		echo '<td class="left"><a href="javascript:add_loc()">Add</a></td>'."\n";
		echo '</tr></tfoot>'."\n";
		echo '</table>'."\n";
		echo '<input type="hidden" id="default_loc" name="WIsettings[default_location]" value="' . $WI->settings['default_location'] . '" />'."\n";
		echo '</fieldset>'."\n";

		echo '<p class="submit">'."\n";
		echo '<span class="submit"><input name="WIclear_cache" value="' . __('Clear Cache', 'WI') . '" type="submit" /></span>'."\n";
		echo '<span class="submit"><input name="WIreset" value="' . __('Reset to Defaults', 'WI') . '" type="submit" /></span>'."\n";
		echo '<span class="submit"><input name="WIsubmit" value="' . __('Update Settings', 'WI') . '" type="submit" /></span>'."\n";
		echo '</p>'."\n";
		echo '</form></div></div>'."\n";

		echo '<div class="wrap">'."\n";
		echo '<h2>' . __('WeatherIcon Usage Information', 'WI') . '</h2>'."\n";
		echo '<ol>'."\n";
		echo '<li> <strong>TODO: Add screenshots and/or instructions for activation via WordPress Sidebar</strong> </li>'."\n";
		echo '<li> <strong>TODO: Add screenshots and/or instructions for using WP-Dash plugin</strong> </li>'."\n";
		echo '<li><p>'. __('<strong>To Add Manually</strong>: add a snippet of code like this wherever you want the weather icon to appear (usually sidebar.php of your template):', 'WI') . '</p>'."\n";
		echo '<code>'."\n";
		echo '&lt;?php if (function_exists(\'WeatherIcon\')) : ?&gt;<br />'."\n";
		echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;li&gt;Regina, Saskatchewan<br/>'."\n";
		echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;ul&gt;&lt;?php WeatherIcon(); ?&gt;&lt;/ul&gt;<br />'."\n";
		echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/li&gt;<br />'."\n";
		echo '&lt;?php endif; ?&gt;'."\n";
		echo '</code><br/><br/>'."\n";
		echo '<ol type="a">'."\n";
		echo '<li><p><strong>Function Parameter Reference</strong></p>'."\n";
		echo '<dd>WeatherIcon( <i>ARGUMENTS_STRING</i>, <i>OPTION_ARRAY</i>);</dd>'."\n";
		echo '<li><p>'. __('Description of the <i>ARGUMENTS_STRING</i> parameters (see the <a href="http://codex.wordpress.org/Template_Tags/How_to_Pass_Tag_Parameters#Tags_with_query-string-style_parameters">WordPress Codex</a> for information on how to pass arguments to a function in a query string format).', 'WI') . '</p>' . "\n";
		echo '<dl>'."\n";
		echo '<dt><strong>location</strong>'."\n";
		echo '<dd>' . __('(<i>string</i>) The name of a location preset (see the Preset Locations section above for a list of valid names).  Defaults to default preset location.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>station</strong>'."\n";
		echo '<dd>' . __('(<i>string</i>) The 4-letter <acronym title="International Civil Aviation Organization">ICAO</acronym> airport indentifier code to display the weather for. Find a location code here: <a href="http://weather.gov/tg/siteloc.shtml">http://weather.gov/tg/siteloc.shtml</a>. Defaults to ICAO of the specified location.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>latitude</strong>'."\n";
		echo '<dd>' . __('(<i>string</i>) The latitude value used to calculate sunrise/sunset times.  Defaults to the location\'s latitude.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>longitude</strong>'."\n";
		echo '<dd>' . __('(<i>string</i>) The longitude value used to calculate sunrise/sunset times.  Defaults to the location\'s longitude.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>gmt_offset</strong>'."\n";
		echo '<dd>' . __('(<i>string</i>) The gmt offset is used to calculate sunrise/sunset times.  Defaults to the location\'s gmt offset.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>dst</strong>'."\n";
		echo '<dd>' . __('(<i>boolean</i>) The dst indicator is used to calculate sunrise/sunset times.  Defaults to the location\'s dst indicator.  Valid values are <tt>TRUE</tt> or <tt>FALSE</tt>.', 'WI').'</dd>'."\n";
		echo '<dt><strong>language</strong></dt>'."\n";
		echo '<dd>' . __('(<i>string</i>) The language in which to display plugin.   The language must match the name of the mo file in the languages directory w/o the extension. IE: deutsch.mo would be simply language=deutsch.  Defaults to the language of the specified location.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>before</strong></dt>'."\n";
		echo '<dd>' . __('(<i>string</i>) Text to display before each weather item description (i.e. the icon, the temperature, the wind, etc.). Defaults to <tt>&lt;li&gt;</tt>.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>after</strong></dt>'."\n";
		echo '<dd>' . __('(<i>string</i>) Text to display after each weather item description (i.e. the icon, the temperature, the wind, etc.). Defaults to <tt>&lt;/li&gt;</tt>.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>header_before</strong></dt>'."\n";
		echo '<dd>' . __('(<i>string</i>) Text to display before the location name.  Defaults to <tt>&lt;b&gt;</tt>.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>header_after</strong></dt>'."\n";
		echo '<dd>' . __('(<i>string</i>) Text to display after the location name.  Defaults to <tt>&lt;/b&gt;</tt>.', 'WI') . '</dd>'."\n";
		echo '<dt><strong>debug</strong></dt>'."\n";
		echo '<dd>' . __('(<i>boolean</i>) Overrides global debug values.  If true, all debug information (cache &amp; condition) will be displayed,  If false, all debug information will be supressed. Valid values are <tt>TRUE</tt> or <tt>FALSE</tt>.', 'WI').'</dd>'."\n";
		echo '<dt><strong>showall</strong></dt>'."\n";
		echo '<dd>' . __('(<i>boolean</i>) Overrides global display options.  If true, all output options will be activate.  If false, the normal display options will be used (in the order specified).  Valid values are <tt>TRUE</tt> or <tt>FALSE</tt>.', 'WI').'</dd>'."\n";
		echo '<dt><strong>showAllLocations</strong></dt>'."\n";
		echo '<dd>' . __('(no value required) Loops through all defined locations and outputs their weather information in order.', 'WI').'</dd>'."\n";
		echo '<dt><strong>display</strong></dt>'."\n";
		echo '<dd>' . __('(<i>boolean</i>) Sets whether to echo or print the function\'s results. Valid values are <tt>TRUE</tt> or <tt>FALSE</tt>.', 'WI') . '</dd>'. "\n";
		echo '</dl>'."\n";
		echo '</li>'."\n";
		echo '<li><p>' . __('Description of the <i>OPTIONS_ARRAY</i> parameters.  Parameters can be set like <tt>array(\'parameter1\'. \'parameter2\', etc)</tt> and their order in the array is the order in which they are displayed.  Valid parameters are:', 'WI') . '</p>' . "\n";
		echo '<dt><strong>display_locname</strong></dt>'."\n";
		echo '<dt><strong>display_icon</strong></dt>'."\n";
		echo '<dt><strong>display_temp</strong></dt>'."\n";
		echo '<dt><strong>display_heatindex</strong></dt>'."\n";
		echo '<dt><strong>display_windchill</strong></dt>'."\n";
		echo '<dt><strong>display_humidex</strong></dt>'."\n";
		echo '<dt><strong>display_dewpt</strong></dt>'."\n";
		echo '<dt><strong>display_wind</strong></dt>'."\n";
		echo '<dt><strong>display_humidity</strong></dt>'."\n";
		echo '<dt><strong>display_barometer</strong></dt>'."\n";
		echo '<dt><strong>display_conditions</strong></dt>'."\n";
		echo '<dt><strong>display_clouds</strong></dt>'."\n";
		echo '<dt><strong>display_cloudsdetailed</strong></dt>'."\n";
		echo '<dt><strong>display_visibility</strong></dt>'."\n";
		echo '<dt><strong>display_updatetime</strong></dt>'."\n";
		echo '<dt><strong>display_sunrise</strong></dt>'."\n";
		echo '<dt><strong>display_sunset</strong></dt>'."\n";
		echo '</li></ol>'."\n";
		echo '</li></ol>'."\n";
		echo '</div>'."\n";

		echo '<div id="overlay" style="display:none"></div>';
	    echo '<div id="box" style="display:none">';
		echo '<img id="close" src="'.WIdirWeb.'/css/mf_lightbox/close.gif" onClick="Lightbox.hideBox()" alt="Close" title="Close this window" />';
	    echo '<div id="boxContents"></div>';
		echo '</div>';
	}

	function get_i18n_options($default_lang)
	{
		global $WI;

		$languages = $WI->get_i18n_list();
	
		$retval = '';
	
		asort($languages);
		foreach ($languages as $language)
		{
			$retval .= '<option value="' . $language . '"' . $WI->setting_selected($default_lang, $language) . '>' . ucwords(preg_replace("/-([0-9]*)-/", "&#$1;", $language)) . '</option>'."\n";
		}
	
		return $retval;
	}
}

// Admin menu items
add_action('admin_menu', array('WeatherIconConfig', 'init'));

?>
