<?php
class WeatherIconCore
{
	var $settings;
	var $icon_packs;
	var $updateURL;
	var $status;
	var $display_errors;

	function WeatherIconCore()
	{
		// Constructor, clear variables
		$this->settings = array();
		$this->status = array();

		// Capture display errors setting
		$this->display_errors = ini_get('display_errors')?ini_get('display_errors'):off;
		$this->status[] = $this->gmtime() . " PHP INI option 'display errors' is " . $this->display_errors;

		$this->disableDisplayErrors();

		// Setup the icon packs
		$this->icon_packs = $this->get_icon_packs();

		// For Wordpress 2.0 compability, we have to delete our
		// cached data item in order to allow updates from the 
		// our configuration page and version upgrades.
		// -Garett
		if (function_exists('wp_cache_delete'))
		{
			wp_cache_delete('WIsettings', 'options');
			wp_cache_delete('WIversion', 'options');
		}

		$this->checkForUpgrade();
		$this->updateSettings();

		$this->restoreDisplayErrors();
	}

	function subchop ( $str, $sub )
	{
		return substr( $str, 0, strlen( $str ) - strlen( $sub ) );
	}

	function endsWith( $str, $sub )
	{
		return ( substr( $str, strlen( $str ) - strlen( $sub ) ) === $sub );
	}

	function disableDisplayErrors()
	{
		if ($this->display_errors == on)
		{
			$this->status[] = $this->gmtime() . " Disabling PHP 'display_errors'";
			ini_set('display_errors', off);
		}
	}

	function restoreDisplayErrors()
	{
		$current_display_errors = ini_get('display_errors')?ini_get('display_errors'):off;

		if ($current_display_errors != $this->display_errors)
		{
			// restore system error preferences
			$this->status[] = $this->gmtime() . " Restoring system PHP 'display_errors' setting";
			ini_set('display_errors', $this->display_errors);
		}
	}

	// Boolean return, checks database to see if the 
	// metar_cache table exists;
	function dbExists()
	{
		global $wpdb;

		if ($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix."metar_cache'") == $wpdb->prefix . "metar_cache")
			return true;

		$this->status[] = $this->gmtime() . " required database table " . $wpdb->prefix . "metar_cache does not exist.";

		return false;
	}

	function dropTable($tablename)
	{
		global $wpdb, $EZSQL_ERROR;

		// Determine the current error count
		$err_cnt = count($EZSQL_ERROR);

		$wpdb->hide_errors();

		$q = <<<SQL
		DROP TABLE IF EXISTS `${tablename}`;
SQL;
		$wpdb->query($q);

		$wpdb->show_errors();
		
		// Did an error occur?
		if ($EZSQL_ERROR[$err_cnt])
		{
			$this->status[] = $this->gmtime() . " database error occured while drop table ${tablename}\n"
							  . "Query: " . $EZSQL_ERROR[$err_cnt]['query'] . "\n"
							  . "Error: " . $EZSQL_ERROR[$err_cnt]['error_str'];
		}
	}

	function createMetarCacheTable()
	{
		global $wpdb, $EZSQL_ERRORf;

		// Workaround (can't reference like ${wpdb->prefix}
		$prefix = $wpdb->prefix;

		// Ensure that the old metar_cache table is killed
		// in case they were running 2.3.1 with Wordpress 2.0+
		$this->dropTable("metar_cache");
		$this->dropTable($prefix . "metar_cache");

		// Determine the current error count
		$err_cnt = count($EZSQL_ERROR);

		$wpdb->hide_errors();

		$q = <<<SQL
		CREATE TABLE IF NOT EXISTS `${prefix}metar_cache` (
			`station` varchar(4) NOT NULL,
			`metar` longtext NOT NULL,
			`expiry` datetime NOT NULL,
			PRIMARY KEY  (`station`)
		);
SQL;

		$wpdb->query($q);
		$wpdb->show_errors();
		
		// Did an error occur?
		if ($EZSQL_ERROR[$err_cnt])
		{
			$this->status[] = $this->gmtime() . " database error occured while creating cache table\n"
							  . "Query: " . $EZSQL_ERROR[$err_cnt]['query'] . "\n"
							  . "Error: " . $EZSQL_ERROR[$err_cnt]['error_str'];
		}		
	}
	
	function clearMetarCacheTable()
	{
		global $wpdb, $EZSQL_ERRORf;

		// Used to clear the table, now just recreates it.
		$this->createMetarCacheTable();
	}	

	function checkForUpgrade() 
	{
		global $wpdb;

		// Let's get the version number from the database.
		// If it is pre 3.0.0 or a new install, it will be
		// automagically set to 0.
		// -Garett
		$WIversionDB = $this->version_num(get_option('WIversion'));

		// if version in DB matches version in plugin, exit (no upgrade req'd)
		if ($WIversionDB == $this->version_num(WIversion)) return;

		$this->status[] = $this->gmtime() . " Version number (from db) is " . $WIversionDB;

		// Check to see if the version # is less than or equal to 2.3.2
		// (our last production version), upgrade the tables (if they
		// don't exist) and upgrade settings.
		// -Garett
		if ($WIversionDB <= 20302)
		{
			// recreate the metar cache table ...
			$this->createMetarCacheTable();

			// Upgrade pre-3.0.0 settings (and options)
			if (get_settings('weather_settings') || get_settings('weather_options'))
			{
				$oldsettings = get_settings('weather_settings');
				$oldoptions = get_settings('weather_options');
				
				$newsettings = array();
				
				foreach ($oldsettings as $index => $val)
				{
					switch ($index)
					{
						case 'station':
							$newsettings['locations']['default']['station'] = $val;
							$newsettings['locations']['default']['latitude'] = '0.0000';
							$newsettings['locations']['default']['longitude'] = '0.0000';
							$newsettings['locations']['default']['gmt_offset'] = get_option('gmt_offset');
							$newsettings['locations']['default']['dst'] = true;
							$newsettings['default_location'] = 'default';
							break;
						// Upgrade ICON LOCATION to ICON PACK.
						case 'icon_location':
						
							// Since there is no way to determine the 
							// matching icon pack, set to default
							$newsettings['icon_pack'] = 'default';
							break;
							
						// Determine ICON EXTENSION
						case 'icon_ext':
						
							// By default, go with default icon set of icon pack (which is 0)
							$icon_ext = 0;
				
							// Get icon pack data and scan for matching extension
							$icon_pack_data = $this->icon_packs['default'];
							foreach ($icon_pack_data['Formats'] as $format_idx => $format)
							{
								// If found, use index value
								if (stristr($val, $format)) $icon_ext = $format_idx;
							}
	
							// Set new index info
							$newsettings['icon_ext'] = $icon_ext;
							break;

						// No longer needed, no file cache support in 3.0.0
						case 'cache_type':
							break;
						
						// No longer needed, no file cache support in 3.0.0
						case 'cache_path':
							break;
						
						// All measurement configuration is stored under 'units' now.
						case 'temp':
							$newsettings['units']['temp'] = $val;
							break;
							
						case 'wind_units':
							$newsettings['units']['wind'] = $val;
							break;

						case 'language':
							$newsettings['language'] = $val;
							$newsettings['locations']['default']['language'] = $val;
							
						case 'visibility_units':
							$newsettings['units']['visibility'] = $val;
							
							// Use visibilty settings to determine the
							// best setting for pressure and 
							// cloud height
							//
							// I assume the imperial system uses inHg for pressure 
							// since the last plugin used inHg by default and had
							// hPa in brackets.
							if ($val == 'km')
							{
								$newsettings['units']['clouds'] = 'm';
								$newsettings['units']['pressure'] = 'hPa';
							}
							else
							{
								$newsettings['units']['clouds'] = 'ft';
								$newsettings['units']['pressure'] = 'inHg';
							}
							
							break;
							
						// If not one of these, it hasn't changed ... so save value
						default:
							$newsettings[$index] = $val;
					}
				}

				foreach ($oldoptions as $val)
				{
					switch ($val)
					{
						// We are no longer getting sunrise/sunset from WeatherUnderground
						// However, we added an option to control the condition
						// notifications so we'll add that here
						case 'fetch_suns':
							$newsettings['display_options'][] = 'display_conditions';
							break;

						// We've renamed metartime
						case 'display_metartime':
							$newsettings['display_options'][] = 'display_updatetime';
							break;
							
						default:
							$newsettings['display_options'][] = $val;
					}
				}
				
				$newsettings['display_options'][] = 'show_current_weather_at_post';

				$newsettings['default_time'] = get_settings('time_format');
				$newsettings['default_date'] = get_settings('date_format');

				// save new settings in new names
				update_option('WIsettings', $newsettings);

				// delete old settings
				delete_option('weather_settings');
				delete_option('weather_options');

				// Wordpress 2.0 compatibility stuff ...
				// I hate the cache ...
				if (function_exists('wp_cache_delete'))
				{
					wp_cache_delete('weather_settings','options');
					wp_cache_delete('weather_options','options');
				}
			}
			else
			{
				// If we can't find old settings, assume new install and reset to defaults	
				update_option('WIsettings', array(
												'locations' => array(
													'Regina SK CA' => array (
														'station' => 'CYQR',
														'latitude' => '50.4333',
														'longitude' => '-104.4667',
														'gmt_offset' => get_option('gmt_offset'),
														'dst' => false,
														'language' => 'english'
												)), 
												'default_location' => 'Regina SK CA',
												'language' => 'english',
												'icon_pack' => 'default',
												'icon_ext' => 0,
												'units' => array (
															'temp' => 'C',
															'wind' => 'km/h', 
															'visibility' => 'km',
															'clouds' => 'm',
															'pressure' => 'hPa'
												),
												'usetime' => 'currenttime',
												'sunrise_hour' => '6',
												'sunset_hour' => '19',
												'cache_timeout' => 1800,
												'default_date' => get_settings('date_format'),
												'default_time' => get_settings('time_format'),
												'display_options' => array (
																		'display_icon',
																		'display_temp',
																		'display_humidity',
																		'display_wind',
																		'display_dewpt',
																		'display_visibility',
																		'display_clouds',
																		'display_conditions',
																		'display_barometer',
																		'display_windchill',
																		'display_humidex',
																		'allow_report'
												),
											));
			}	

			update_option('WIversion', WIversion);
			return;
		}				

		// removed upgrade code for version less than 2.99.79
		// if using a version older than version 2.99.79, you will
		// have to upgrade to version 111 first, then to 2.99.79

		if ($WIversionDB < 299124)
		{
			$oldsettings = get_settings('WIsettings');

			if (in_array('display_loc_name', $oldsetting['display_options']))
			{
				$oldsettings['display_options'][] = 'display_locname';
			}

			if (in_array('display_clouds_detailed', $oldsetting['display_options']))
			{
				$oldsettings['display_options'][] = 'display_cloudsdetailed';
			}

			// We should remove the old variables, but they are only wasting some space
			// and will be wiped out the second you save settings on the configuration
			// screen so there really isn't a point to doing it.
			//
			// In any case, save it
			update_option('WIsettings', $oldsettings);
		}

		// Drop and recreate the Cache Table
		$this->clearMetarCacheTable();

		// Save current version
		update_option('WIversion', WIversion);
	}

	function checkForNewVersion()
	{
		$this->status[] = $this->gmtime() . " Checking " . WIRemoteHost . " for new version of plugin.";
		
		$client = new Snoopy();
		$client->rawheaders["Pragma"] = "no-cache";
		$client->read_timeout = 15;
		
		// Fetch latest version information from plugin rep
		if ($client->fetchtext("http://" . WIRemoteHost . WIRemoteVersion))
		{
			$this->status[] = $this->gmtime() . " " . WIRemoteHost . " responded " . trim($client->response_code);
			$this->status[] = $this->gmtime() . " " . WIRemoteHost . " response data\n" . trim($client->results);
			
			// We got a good response, let's parse it and hope everything is good
			if (strpos($client->response_code, '200 '))
			{
				$tmp = explode("\n", $client->results);
				
				$server_version = $tmp[0];
				$this->updateURL = $tmp[1];
				
				$this->status[] = $this->gmtime() . " Current plugin version is " . $server_version;
				$this->status[] = $this->gmtime() . " Update URL is " . $this->updateURL;
				
				// We (hope) we have the latest version
				// return it
				return intval($server_version);
			}
		}
		else
		{
			$this->status[] = $this->gmtime() . " Snoopy error while checking for new plugin version " . $client->error;
		}
		
		// On failure, assume no upgrade
		return $this->version_num(WIversion);		
	}

	function getUpdateURL()
	{
		return $this->updateURL;
	}

	function ReportBrokenIcon($path,$icon_name, $icon_pack)
	{
		$parameters = array (
			'IconPath' => $path,
			'IconName' => $icon_name,
			'IconPack' => $icon_pack
		);

		$this->status[] = $this->gmtime() . " Reporting Broken Icon.";
		
		$client = new Snoopy();
		$client->rawheaders["Pragma"] = "no-cache";
		$client->read_timeout = 15;
		
		if ($client->submit("http://" . WIRemoteHost . WIRemoteReport, $parameters))
		{
			$this->status[] = $this->gmtime() . " Kapla!  Broken icon reported.";
		}
		else
		{
			$this->status[] = $this->gmtime() . " Snoopy error while remoting broken icon.  " . $client->error;
		}
	}
	
	function updateSettings()
	{
		$this->settings = get_settings('WIsettings');
	}

	function get_i18n_list()
	{
		$languages = array();
		$languages[] = "english";
	
		$handler = opendir( WIi18ndir );
	
		while ($file = readdir($handler)) {
			if (strpos($file, '.mo')) {
				$languages[] = substr($file, 0, strlen($file) - 3);
			}
		}
	
		closedir($handler);
	
		return $languages;
	}	

	function get_icon_pack_data($WIicon_pack_def)
	{
		$WIicon_pack_data = implode('', file(WIicondir . $WIicon_pack_def . "/.icon.ipk"));
		preg_match("|Icon Pack Name:(.*)|i", $WIicon_pack_data, $icon_pack_name);
		preg_match("|Icon Pack URI:(.*)|i", $WIicon_pack_data, $icon_pack_uri);
		preg_match("|Description:(.*)|i", $WIicon_pack_data, $description);
		preg_match("|Designer:(.*)|i", $WIicon_pack_data, $designer_name);
		preg_match("|Designer URI:(.*)|i", $WIicon_pack_data, $designer_uri);
		preg_match("|Formats:(.*)|i", $WIicon_pack_data, $formats);
	
		$description = wptexturize(trim($description[1]));
		$name = trim($icon_pack_name[1]);
		$icon_pack = $name;
		if ('' != $icon_pack_uri[1] && '' != $name)
		{
			$icon_pack = '<a href="' . trim($icon_pack_uri[1]) . '" title="' . __('Visit icon pack homepage', 'WI') . '">' . $icon_pack . '</a>';
		}
	
		if ('' == $designer_uri[1])
		{
			$designer = trim($designer_name[1]);
		}
		else
		{
			$designer = '<a href="' . trim($designer_uri[1]) . '" title="' . __('Visit designer homepage', 'WI') . '">' . trim($designer_name[1]) . '</a>';
		}
	
		$formats = explode(' ', trim($formats[1]));
	
		return array('Name' => $WIicon_pack_def, 'Title' => $icon_pack, 'Description' => $description, 'Designer' => $designer, 'Formats' => $formats);
	}

	function get_icon_packs()
	{
        $options = array(
            XML_UNSERIALIZER_OPTION_COMPLEXTYPE => 'array',
            XML_UNSERIALIZER_OPTION_ATTRIBUTE_KEY => array(
              'icon'      => 'name',
              'condition' => 'code'),
            XML_UNSERIALIZER_OPTION_ATTRIBUTES_PARSE    => false,
            XML_UNSERIALIZER_OPTION_ATTRIBUTES_ARRAYKEY => false,
            XML_UNSERIALIZER_OPTION_FORCE_ENUM => array ('format')
        );

        $unserializer = &new XML_Unserializer($options);

        $WIicon_packs = array();
        $WIicon_dir = @ dir(WIicondir);
   
        if ($WIicon_dir)
        {
            while(($file = $WIicon_dir->read()) !== false)
            {
                if (is_file(WIicondir . $file) && $this->endsWith($file, '.ipk'))
                {
                    $tar = new Archive_Tar(WIicondir . $file);
                    $xml = $tar->extractInString('meta.xml');
                    $status = $unserializer->unserialize($xml, false);
					$ipkName = $this->subchop($file, '.ipk');

                    if (PEAR::isError($status))
                    {
						$this->status[] = $this->gmtime() . ' icon pack ' . $ipkName . ' is invalid.';	
                    }
                    else
                    {
						$this->status[] = $this->gmtime() . ' icon pack ' . $ipkName . ' is valid.';
                        $WIicon_packs[$ipkName] = $unserializer->getUnserializedData();
                    }
                }
            }
        }

		return $WIicon_packs;
	}

	function setting_checked($index, $value)
	{
		if (strpos($index, '|'))
		{
			$values = preg_split('/\|/', $index);

			$index = $this->settings;
			foreach ($values as $val)
			{
				$index = $index[$val];
			}
		}
		else
		{
			$index = $this->settings[$index];
		}

		if ($index == $value) return 'checked="checked"';
	}
	
	function setting_selected ($index, $value)
	{
		if (strpos($index, '|'))
		{
			$values = preg_split('/\|/', $index);

			$index = $this->settings;
			foreach ($values as $val)
			{
				$index = $index[$val];
			}
		}
		else
		{
			$index = $this->settings[$index];
		}

		if ($index == $value) return 'selected="selected"';
	}
	
	function unit_selected ($index, $value)
	{
		if ($this->settings['units'][$index] == $value) return 'selected="selected"';
	}	
	
	function option_checked ($index)
	{
		if (in_array($index, $this->settings['display_options'])) return 'checked="checked"';
	}

	function version_num ($ver)
	{
		list($major, $minor, $patch) = split('\.', $ver, 3);
		return intval(($major * 100000) + ($minor * 1000) + $patch);
	}
	
	function gmtime()
	{
		return time() - date('Z', time());
	}
	
	function cache_expiry()
	{
		return gmdate("Y-m-d H:i:s", time() + $this->settings['cache_timeout']);
	}
	
	function cache_metar($station, $metar, $cached=FALSE)
	{
		global $wpdb, $EZSQL_ERROR;
		
		if ($station == substr($metar,0,4))
		{
			// Determine the current error count
			$err_cnt = count($EZSQL_ERROR);

			// Insert data into database
			$wpdb->hide_errors();
			if ($cached)
			{
				// station already cached, update information
				$wpdb->query("UPDATE `" . $wpdb->prefix . "metar_cache` SET metar='${metar}', expiry='" . $this->cache_expiry() . "' WHERE station='${station}'");
			}
			else
			{
				// new station, insert new row
				$wpdb->query("INSERT INTO `" . $wpdb->prefix . "metar_cache` (station, metar, expiry) VALUES ('${station}', '${metar}', '" . $this->cache_expiry() . "')");
			}
			$wpdb->show_errors();
			
			// Did an error occur?
			if ($EZSQL_ERROR[$err_cnt])
			{
				$this->status[] = $this->gmtime() . " database error occured while saving the cache\n"
								  . "Query: " . $EZSQL_ERROR[$err_cnt]['query'] . "\n"
								  . "Error: " . $EZSQL_ERROR[$err_cnt]['error_str'];
				
				return FALSE;
			}
			else
			{
				// return metar
				return $metar;
			}
		}
		else
		{
			// metar is invalid, drop it
			$this->status[] = $this->gmtime() . " Retrieved metar is invalid";
			return FALSE;
		}
	}
	
	function fetch_metar($station)
	{
		global $wpdb;
		
		$metar = "";
		$cached = FALSE;
		
		$station = strtoupper($station);
		
		// Check the cache
		$cache = $wpdb->get_row ("SELECT metar, expiry FROM `" . $wpdb->prefix . "metar_cache` WHERE station = '${station}'", ARRAY_A);
		if ($cache)
		{
			// check to see if it is valid
			if ($cache['expiry'] >= gmdate("Y-m-d H:i:s", time()))
			{
				// record status (for debugging)
				$this->status[] = $this->gmtime() . " Cache valid, retrieved metar " . $cache['metar'];
				return $cache['metar'];
			}
			else
			{
				$this->status[] = $this->gmtime() . " Cache expired";
			}
			$cached = TRUE;
		}
		else
		{
			$this->status[] = $this->gmtime() . " No cached metar for station ${station}";
		}
		
		$client = new Snoopy();
		$client->rawheaders["Pragma"] = "no-cache";
		$client->read_timeout = 15;
	
		if (WIuseNOAA)
		{
			// Fetch from NOAA (IWS) -- Internet Weather Service
			// See http://weather.noaa.gov/weather/metar.shtml
			if ($client->fetchtext("http://weather.noaa.gov/pub/data/observations/metar/stations/${station}.TXT"))
			{
				$this->status[] = $this->gmtime() . " NOAA (IWS) responded " . trim($client->response_code);
				$this->status[] = $this->gmtime() . " NOAA (IWS) response data\n" . trim($client->results);
				
				// We got a good response, let's parse it and hope everything is good
				if (strpos($client->response_code, '200 '))
				{
					// annoyingly one can not do just do 
					//	 explode("\n", $client->results)[1]
					$tmp = explode("\n", $client->results);
	
					$metar = $this->cache_metar($station, $tmp[1], $cached);
	
					// if valid, return
					if ($metar) return $metar;
				}
			}
			else
			{
				$this->status[] = $this->gmtime() . " Snoopy error while fetching metar from NOAA (IWS) " . $client->error;
			}
		}
		// We don't have a metar, let's try ADDS.
		$this->status[] = $this->gmtime() . " Trying NOAA's ADDS metar repository.";
	
		if (WIuseADDS)
		{
			// Fetch from NOAA (ADDS) -- Aviation Digital Data Service
			// See http://adds.aviationweather.gov/metars/
			if ($client->fetchtext("http://adds.aviationweather.gov/metars/index.php?station_ids=${station}&std_trans=standard&chk_metars=on&hoursStr=most+recent+only&submitmet=Submit"))
			{
				$this->status[] = $this->gmtime() . " NOAA (ADDS) responded " . trim($client->response_code);
				$this->status[] = $this->gmtime() . " NOAA (ADDS) response data:\n" . trim($client->results);
				
				// annoyingly one can not do just do 
				//	 explode("\n", $client->results)[5]
				$tmp = explode("\n", $client->results);
				$metar = $this->cache_metar($station, $tmp[5], $cached);
				
				// if valid, return
				if ($metar) return $metar;
			}
			else
			{
				$this->status[] = $this->gmtime() . " Snoopy error while fetching metar from NOAA (ADDS) " . $client->error;
			}
		}

		// If we are here ... bad things have happened
		if ($cached)
		{
			// Thank god we have cached data!  Use it!
			$this->status[] = $this->gmtime() . " No valid metar data avaliable, using expired cache. METAR " . $cache['metar'];
			return $cache['metar'];
		}
		else
		{
			$this->status[] = $this->gmtime() . " FATAL ERROR!  NO METAR AVAILABLE!";
			return FALSE;
		}
	}

	function bool ($value)
	{
		switch (strtoupper($value))
		{
			case "FALSE":	return FALSE;	break;
			case "0":		return FALSE;	break;
			case "TRUE":	return TRUE;	break;
			case "1":		return TRUE;	break;
			default:		return FALSE;
		}
	}
}
?>
